# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hub.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\thub.proto\x12\x06hubapi\x1a\x1fgoogle/protobuf/timestamp.proto\"I\n\x18RunCodeCollectorsRequest\x12-\n\x13\x63ommit_max_age_secs\x18\x01 \x01(\x03R\x10\x63ommitMaxAgeSecs\"M\n\x11\x43odeCollectionRun\x12\x19\n\x08repo_uri\x18\x01 \x01(\tR\x07repoUri\x12\x1d\n\ncommit_sha\x18\x02 \x01(\tR\tcommitSha\"h\n\x19RunCodeCollectorsResponse\x12K\n\x14\x63ode_collection_runs\x18\x01 \x03(\x0b\x32\x19.hubapi.CodeCollectionRunR\x12\x63odeCollectionRuns\"F\n\x0e\x43ollectRequest\x12\x34\n\x0b\x63ollections\x18\x01 \x03(\x0b\x32\x12.hubapi.CollectionR\x0b\x63ollections\"\x11\n\x0f\x43ollectResponse\"2\n\x13PullManifestRequest\x12\x1b\n\tmain_repo\x18\x01 \x01(\tR\x08mainRepo\".\n\x14PullManifestResponse\x12\x16\n\x06status\x18\x01 \x01(\tR\x06status\"\x9e\x03\n\nCollection\x12#\n\rcollection_id\x18\x01 \x01(\tR\x0c\x63ollectionId\x12\x12\n\x04\x62lob\x18\x02 \x01(\tR\x04\x62lob\x12;\n\x0bobserved_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\nobservedAt\x12!\n\x0c\x63ollector_id\x18\x04 \x01(\tR\x0b\x63ollectorId\x12\x30\n\x06source\x18\x05 \x01(\x0e\x32\x18.hubapi.CollectionSourceR\x06source\x12\x42\n\ndimensions\x18\x06 \x03(\x0b\x32\".hubapi.Collection.DimensionsEntryR\ndimensions\x12!\n\x0c\x63omponent_id\x18\x07 \x01(\tR\x0b\x63omponentId\x12\x1f\n\x0b\x64uration_ms\x18\x08 \x01(\x03R\ndurationMs\x1a=\n\x0f\x44imensionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\x17\n\x15\x43urrentVersionRequest\"M\n\x16\x43urrentVersionResponse\x12\x18\n\x07version\x18\x01 \x01(\tR\x07version\x12\x19\n\x08repo_uri\x18\x02 \x01(\tR\x07repoUri\"\x88\x01\n\x1b\x43heckComponentStatusRequest\x12!\n\x0c\x63omponent_id\x18\x01 \x01(\tR\x0b\x63omponentId\x12\x10\n\x03sha\x18\x02 \x01(\tR\x03sha\x12\x34\n\x04type\x18\x03 \x01(\x0e\x32 .hubapi.CheckComponentStatusTypeR\x04type\".\n\x1c\x43heckComponentStatusResponse\x12\x0e\n\x02ok\x18\x01 \x01(\x08R\x02ok\"\xb0\x01\n\x0f\x41ssertionResult\x12\x1a\n\x02op\x18\x01 \x01(\x0e\x32\n.hubapi.OpR\x02op\x12\x12\n\x04\x61rgs\x18\x02 \x03(\tR\x04\x61rgs\x12+\n\x06result\x18\x03 \x01(\x0e\x32\x13.hubapi.CheckStatusR\x06result\x12,\n\x0f\x66\x61ilure_message\x18\x04 \x01(\tH\x00R\x0e\x66\x61ilureMessage\x88\x01\x01\x42\x12\n\x10_failure_message\"l\n\x14\x43ollectPolicyRequest\x12\x1b\n\tpolicy_id\x18\x01 \x01(\tR\x08policyId\x12\x37\n\nassertions\x18\x02 \x03(\x0b\x32\x17.hubapi.AssertionResultR\nassertions\"1\n\x15\x43ollectPolicyResponse\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success*g\n\x10\x43ollectionSource\x12\x1d\n\x19\x43OLLECTION_SOURCE_UNKNOWN\x10\x00\x12\x18\n\x14\x43OLLECTION_SOURCE_CI\x10\x01\x12\x1a\n\x16\x43OLLECTION_SOURCE_CODE\x10\x02*\x90\x01\n\x18\x43heckComponentStatusType\x12\'\n#CHECK_COMPONENT_STATUS_TYPE_UNKNOWN\x10\x00\x12\'\n#CHECK_COMPONENT_STATUS_TYPE_RELEASE\x10\x01\x12\"\n\x1e\x43HECK_COMPONENT_STATUS_TYPE_PR\x10\x02*\x87\x01\n\x0b\x43heckStatus\x12\x18\n\x14\x43HECK_STATUS_UNKNOWN\x10\x00\x12\x15\n\x11\x43HECK_STATUS_PASS\x10\x01\x12\x15\n\x11\x43HECK_STATUS_FAIL\x10\x02\x12\x18\n\x14\x43HECK_STATUS_NO_DATA\x10\x03\x12\x16\n\x12\x43HECK_STATUS_ERROR\x10\x04*\xc4\x01\n\x02Op\x12\x0e\n\nOP_UNKNOWN\x10\x00\x12\x0f\n\x0bOP_CONTAINS\x10\x01\x12\r\n\tOP_EQUALS\x10\x02\x12\x0b\n\x07OP_TRUE\x10\x03\x12\x0c\n\x08OP_FALSE\x10\x04\x12\x0e\n\nOP_GREATER\x10\x05\x12\x17\n\x13OP_GREATER_OR_EQUAL\x10\x06\x12\x0b\n\x07OP_LESS\x10\x07\x12\x14\n\x10OP_LESS_OR_EQUAL\x10\x08\x12\x0c\n\x08OP_MATCH\x10\t\x12\x0b\n\x07OP_FAIL\x10\n\x12\x0c\n\x08OP_ERROR\x10\x0b\x32\xf6\x03\n\x03Hub\x12<\n\x07\x43ollect\x12\x16.hubapi.CollectRequest\x1a\x17.hubapi.CollectResponse\"\x00\x12M\n\x0cPullManifest\x12\x1b.hubapi.PullManifestRequest\x1a\x1c.hubapi.PullManifestResponse\"\x00\x30\x01\x12Z\n\x11RunCodeCollectors\x12 .hubapi.RunCodeCollectorsRequest\x1a!.hubapi.RunCodeCollectorsResponse\"\x00\x12Q\n\x0e\x43urrentVersion\x12\x1d.hubapi.CurrentVersionRequest\x1a\x1e.hubapi.CurrentVersionResponse\"\x00\x12\x63\n\x14\x43heckComponentStatus\x12#.hubapi.CheckComponentStatusRequest\x1a$.hubapi.CheckComponentStatusResponse\"\x00\x12N\n\rCollectPolicy\x12\x1c.hubapi.CollectPolicyRequest\x1a\x1d.hubapi.CollectPolicyResponse\"\x00\x42\nZ\x08./hubapib\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'hub_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\010./hubapi'
  _globals['_COLLECTION_DIMENSIONSENTRY']._options = None
  _globals['_COLLECTION_DIMENSIONSENTRY']._serialized_options = b'8\001'
  _globals['_COLLECTIONSOURCE']._serialized_start=1553
  _globals['_COLLECTIONSOURCE']._serialized_end=1656
  _globals['_CHECKCOMPONENTSTATUSTYPE']._serialized_start=1659
  _globals['_CHECKCOMPONENTSTATUSTYPE']._serialized_end=1803
  _globals['_CHECKSTATUS']._serialized_start=1806
  _globals['_CHECKSTATUS']._serialized_end=1941
  _globals['_OP']._serialized_start=1944
  _globals['_OP']._serialized_end=2140
  _globals['_RUNCODECOLLECTORSREQUEST']._serialized_start=54
  _globals['_RUNCODECOLLECTORSREQUEST']._serialized_end=127
  _globals['_CODECOLLECTIONRUN']._serialized_start=129
  _globals['_CODECOLLECTIONRUN']._serialized_end=206
  _globals['_RUNCODECOLLECTORSRESPONSE']._serialized_start=208
  _globals['_RUNCODECOLLECTORSRESPONSE']._serialized_end=312
  _globals['_COLLECTREQUEST']._serialized_start=314
  _globals['_COLLECTREQUEST']._serialized_end=384
  _globals['_COLLECTRESPONSE']._serialized_start=386
  _globals['_COLLECTRESPONSE']._serialized_end=403
  _globals['_PULLMANIFESTREQUEST']._serialized_start=405
  _globals['_PULLMANIFESTREQUEST']._serialized_end=455
  _globals['_PULLMANIFESTRESPONSE']._serialized_start=457
  _globals['_PULLMANIFESTRESPONSE']._serialized_end=503
  _globals['_COLLECTION']._serialized_start=506
  _globals['_COLLECTION']._serialized_end=920
  _globals['_COLLECTION_DIMENSIONSENTRY']._serialized_start=859
  _globals['_COLLECTION_DIMENSIONSENTRY']._serialized_end=920
  _globals['_CURRENTVERSIONREQUEST']._serialized_start=922
  _globals['_CURRENTVERSIONREQUEST']._serialized_end=945
  _globals['_CURRENTVERSIONRESPONSE']._serialized_start=947
  _globals['_CURRENTVERSIONRESPONSE']._serialized_end=1024
  _globals['_CHECKCOMPONENTSTATUSREQUEST']._serialized_start=1027
  _globals['_CHECKCOMPONENTSTATUSREQUEST']._serialized_end=1163
  _globals['_CHECKCOMPONENTSTATUSRESPONSE']._serialized_start=1165
  _globals['_CHECKCOMPONENTSTATUSRESPONSE']._serialized_end=1211
  _globals['_ASSERTIONRESULT']._serialized_start=1214
  _globals['_ASSERTIONRESULT']._serialized_end=1390
  _globals['_COLLECTPOLICYREQUEST']._serialized_start=1392
  _globals['_COLLECTPOLICYREQUEST']._serialized_end=1500
  _globals['_COLLECTPOLICYRESPONSE']._serialized_start=1502
  _globals['_COLLECTPOLICYRESPONSE']._serialized_end=1551
  _globals['_HUB']._serialized_start=2143
  _globals['_HUB']._serialized_end=2645
# @@protoc_insertion_point(module_scope)
